using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using Mapbox.Unity.Location;
using Mapbox.Unity.Map;
using Mapbox.Utils;
using System.Linq;
using System.IO;
using System;

public class DataPointVisualize : MonoBehaviour
{
  public AbstractMap map;
  public TextAsset data;
  public GameObject dataPathPrefab;
  public int dataLimit = 1000;

  ILocationProvider _locationProvider;
  ILocationProvider LocationProvider
  {
    get
    {
      if (_locationProvider == null)
      {
        _locationProvider = LocationProviderFactory.Instance.DefaultLocationProvider;
      }

      return _locationProvider;
    }
  }

  Vector3 _targetPosition;

  List<DataPath> paths = new List<DataPath>();

  void Start()
  {
    LocationProviderFactory.Instance.mapManager.OnInitialized += () =>  {

      List<DataPath.DataPoint > objs ;
      using (var rdr = new StringReader(this.data.text))
      {
        objs = Sinbad.CsvUtil.LoadObjects<DataPath.DataPoint >(rdr);
      }
    
      //objs.Sort(Comparer<DataPath.DataPoint>.Create((l,r) => l.direction - r.direction));
      //objs.Sort(Comparer<DataPath.DataPoint>.Create((l,r) => isign(l.timestamp - r.timestamp)));
      //objs.Sort(Comparer<DataPath.DataPoint>.Create((l,r) => l.id.GetHashCode() - r.id.GetHashCode()));
      objs.Sort(Comparer<DataPath.DataPoint>.Create((l,r) => l.id.CompareTo(r.id)));

      int lastID = "".GetHashCode();
      DataPath path = null;
      int pathCount = 0;

      for (int i = 0; i < objs.Count; i++)
      {
        var point = objs[i];

        if(lastID != point.id.GetHashCode()) {

          if(path != null) {
            path.Setup();
          }

          if(pathCount == this.dataLimit) break;

          lastID = point.id.GetHashCode();

          //GameObject go = new GameObject(point.id);
          GameObject go = Instantiate(this.dataPathPrefab, this.transform);
          go.name = point.id;
          path = go.GetComponent<DataPath>();
          path.id = point.id;

          paths.Add(path);

          pathCount++;
        }

        path.points.Add(point);
      }

      path.Setup();

      Debug.Log("dataPath count = " + GameObject.FindObjectsOfType<DataPath>().Length);
    };
    
  }

  private int isign(long s)
  {
    if(s < 0) return -1;
    if(s > 0) return 1;
    return 0;
  }

  public void OnGUI()
  {

    // Starts a vertical group
    GUILayout.BeginVertical("box");

    if (GUILayout.Button("Y = Time")) {
      for (int i = 0; i < paths.Count; i++)
      {
        var path = paths[i];
        var lr = path.GetComponent<LineRenderer>();



        int n = lr.positionCount;
        var points = new Vector3[n];
        lr.GetPositions(points);
        for (int j = 0; j != n; ++j) {
          points[j].y = ((float)j / n) * 40;
        }
        lr.SetPositions(points);
      }
    }
    GUILayout.Space(20);
    for (int i = 0; i < paths.Count; i++)
    {
      var path = paths[i];
      bool active = path.GetComponent<LineRenderer>().enabled;
      string text = "[" + (active ? "X" : "_") + "] " + path.id;

      if (GUILayout.Button(text)) {
        path.GetComponent<LineRenderer>().enabled = !active;
      }
    }
    GUILayout.EndVertical();
    
  }
}
